package at.jku.students.graph;

import at.jku.ssw.Iterator;
import at.jku.ssw.graph.Edge;
import at.jku.ssw.graph.Graph;
import at.jku.ssw.graph.Vertex;
import at.jku.ssw.graphviz.DotMaker;
import at.jku.ssw.util.Out;

public class WarshallMain {
	public static void main(String[] args) {
		testVO();
		test1();
		test2();
		testTest();
		testChain();
	}

	static void testChain() {
		Out.println("\n*** Test Chain - transitive closure (directed, unweighted) ***");
		Graph g = new Graph();
		Vertex a = new Vertex('A', true);
		g.addVertex(a);
		Vertex b = new Vertex('B');
		g.addVertex(b);
		Vertex c = new Vertex('C');
		g.addVertex(c);
		Vertex d = new Vertex('D');
		g.addVertex(d);

		g.addEdge(new Edge(a, b, Edge.Kind.Directed));
		g.addEdge(new Edge(b, c, Edge.Kind.Directed));
		g.addEdge(new Edge(c, d, Edge.Kind.Directed));
		makeDot("Chaina.dot", g);
		long[] m = Warshall.graphToMatrix(g);
		System.out.println("<< Initial Matrix >>");
		printMatrix(g, m);

		Warshall.transitiveClosure(m);
		System.out.println("<< Transitive Closure >>");
		printMatrix(g, m);

		Graph closure = Warshall.matrixToGraph(g.vertices, m);
		Out.println(String.format("Warshall: %d edges in graph, %d edges in transitive closure", g.edges.size(),
				closure.edges.size()));
		makeDot("Chainb.dot", closure);
	}

	static void testVO() {
		Out.println("\n*** Test VO - transitive closure (directed, unweighted) ***");
		Graph g = new Graph();
		Vertex a = new Vertex('A', true);
		g.addVertex(a);
		Vertex b = new Vertex('B');
		g.addVertex(b);
		Vertex c = new Vertex('C');
		g.addVertex(c);
		Vertex d = new Vertex('D');
		g.addVertex(d);
		Vertex e = new Vertex('E');
		g.addVertex(e);

		g.addEdge(new Edge(a, b, Edge.Kind.Directed));
		g.addEdge(new Edge(b, c, Edge.Kind.Directed));
		g.addEdge(new Edge(c, a, Edge.Kind.Directed));
		g.addEdge(new Edge(d, a, Edge.Kind.Directed));
		g.addEdge(new Edge(e, d, Edge.Kind.Directed));
		makeDot("VOa.dot", g);
		long[] m = Warshall.graphToMatrix(g);
		System.out.println("<< Initial Matrix >>");
		printMatrix(g, m);

		Warshall.transitiveClosure(m);
		System.out.println("<< Transitive Closure >>");
		printMatrix(g, m);

		Graph closure = Warshall.matrixToGraph(g.vertices, m);
		Out.println(String.format("Warshall: %d edges in graph, %d edges in transitive closure", g.edges.size(),
				closure.edges.size()));
		makeDot("VOb.dot", closure);
	}

	static void testTest() {
		Out.println("\n*** Test VO - transitive closure (directed, unweighted) ***");
		Graph g = new Graph();
		Vertex a = new Vertex('A', true);
		g.addVertex(a);
		Vertex b = new Vertex('B');
		g.addVertex(b);
		Vertex c = new Vertex('C');
		g.addVertex(c);
		Vertex d = new Vertex('D');
		g.addVertex(d);
		Vertex e = new Vertex('E');
		g.addVertex(e);

		g.addEdge(new Edge(a, b, Edge.Kind.Directed));
		g.addEdge(new Edge(b, c, Edge.Kind.Directed));
		g.addEdge(new Edge(c, a, Edge.Kind.Directed));
		g.addEdge(new Edge(d, a, Edge.Kind.Directed));
		g.addEdge(new Edge(e, d, Edge.Kind.Directed));
		g.addEdge(new Edge(a, d, Edge.Kind.Directed));

		makeDot("TESTa.dot", g);
		long[] m = Warshall.graphToMatrix(g);
		System.out.println("<< Initial Matrix >>");
		printMatrix(g, m);

		Warshall.transitiveClosure(m);
		System.out.println("<< Transitive Closure >>");
		printMatrix(g, m);

		Graph closure = Warshall.matrixToGraph(g.vertices, m);
		Out.println(String.format("Warshall: %d edges in graph, %d edges in transitive closure", g.edges.size(),
				closure.edges.size()));
		makeDot("TESTb.dot", closure);
	}

	static void test1() {
		Out.println("\n*** Test 1 - transitive closure (directed, unweighted) ***");
		Graph g = new Graph();
		Vertex a = new Vertex('A', true);
		g.addVertex(a);
		Vertex b = new Vertex('B');
		g.addVertex(b);
		Vertex c = new Vertex('C');
		g.addVertex(c);
		Vertex d = new Vertex('D');
		g.addVertex(d);
		Vertex e = new Vertex('E');
		g.addVertex(e);
		Vertex f = new Vertex('F');
		g.addVertex(f);
		g.addEdge(new Edge(a, b, Edge.Kind.Directed));
		g.addEdge(new Edge(a, d, Edge.Kind.Directed));
		g.addEdge(new Edge(c, f, Edge.Kind.Directed));
		g.addEdge(new Edge(d, b, Edge.Kind.Directed));
		g.addEdge(new Edge(d, e, Edge.Kind.Directed));
		g.addEdge(new Edge(d, f, Edge.Kind.Directed));
		g.addEdge(new Edge(e, c, Edge.Kind.Directed));
		g.addEdge(new Edge(f, e, Edge.Kind.Directed));

		makeDot("1a.dot", g);
		long[] m = Warshall.graphToMatrix(g);
		System.out.println("<< Initial Matrix >>");
		printMatrix(g, m);

		Warshall.transitiveClosure(m);
		System.out.println("<< Transitive Closure >>");
		printMatrix(g, m);

		Graph closure = Warshall.matrixToGraph(g.vertices, m);
		Out.println(String.format("Warshall: %d edges in graph, %d edges in transitive closure", g.edges.size(),
				closure.edges.size()));
		makeDot("1b.dot", closure);
	}

	static void test2() {
		Out.println("\n*** Test 2 - transitive closure (undirected, unweighted) ***");
		Graph g = new Graph();
		Vertex a = new Vertex('A', true);
		g.addVertex(a);
		Vertex b = new Vertex('B');
		g.addVertex(b);
		Vertex c = new Vertex('C');
		g.addVertex(c);
		Vertex d = new Vertex('D');
		g.addVertex(d);
		g.addEdge(new Edge(a, b, 0, Edge.Kind.Undirected));
		g.addEdge(new Edge(b, c, 0, Edge.Kind.Undirected));
		g.addEdge(new Edge(b, d, 0, Edge.Kind.Undirected));
		g.addEdge(new Edge(c, a, 0, Edge.Kind.Undirected));
		makeDot("2a.dot", g);
		long[] m = Warshall.graphToMatrix(g);
		System.out.println("<< Initial Matrix >>");
		printMatrix(g, m);

		Warshall.transitiveClosure(m);
		System.out.println("<< Transitive Closure >>");
		printMatrix(g, m);

		Graph closure = Warshall.matrixToGraph(g.vertices, m);
		Out.println(String.format("Warshall: %d edges in graph, %d edges in transitive closure", g.edges.size(),
				closure.edges.size()));
		makeDot("2b.dot", closure);
	}

	public static void printMatrix(Graph g, long[] m) {
		System.out.print("  ");
		for (Iterator it = g.vertices.iterator(); it.hasNext();) {
			char c = ((Vertex) it.next()).value;
			System.out.print(c + " ");
		}

		int len = g.vertices.size();

		System.out.println();
		Iterator it = g.vertices.iterator();
		for (int i = 0; i < len; i++) {
			System.out.print(it.next() + " ");
			for (int j = 0; j < len; j++) {
				System.out.print(Warshall.isBitSet(m[i], j) ? "1 " : "0 ");
			}
			System.out.println();
		}
	}

	public static void makeDot(String fileName, Graph g) {
		Out.open(fileName);
		Out.println(DotMaker.makeDotForGraph(g));
		Out.close();
	}
}
